@@ -1,7 +1,7 @@
 # mypy: ignore-errors
 #
 # PEP 695 isn't yet supported in mypy. (April 4, 2024)
-"""Type Aliases (3.12+)
+"""Type aliases (3.12+)
 
 In Python 3.12, the :code:`type` statement is introduced to create type aliases.
 
