@@ -1,4 +1,4 @@
-"""Attrs Integration
+"""Attrs integration
 
 In addition to standard dataclasses, :func:`tyro.cli()` also supports
 `attrs <https://www.attrs.org/>`_ classes.
