@@ -2,7 +2,7 @@
 #
 # Passing a Union type directly to tyro.cli() doesn't type-check correctly in
 # mypy. This will be fixed by `typing.TypeForm`: https://peps.python.org/pep-0747/
-"""Subcommands are Unions
+"""Subcommands are unions
 
 All of :mod:`tyro`'s subcommand features are built using unions over struct
 types (typically dataclasses). Subcommands are used to choose between types in
