@@ -1,11 +1,11 @@
-"""Sequenced Subcommands
+"""Sequenced subcommands
 
 Multiple unions over struct types are populated using a series of subcommands.
 
 Usage:
 
-    # We apply the :class:`tyro.conf.ConsolidateSubcommandArgs` flag. This
-    # pushes all arguments to the end of the command:
+    # We apply the :class:`tyro.conf.CascadeSubcommandArgs` flag. This
+    # allows arguments to be intermixed more flexibly:
     python ./03_multiple_subcommands.py --help
     python ./03_multiple_subcommands.py dataset:mnist --help
     python ./03_multiple_subcommands.py dataset:mnist optimizer:adam --help
@@ -32,6 +32,7 @@ class Mnist:
 class ImageNet:
     subset: Literal[50, 100, 1000]
     """Choose between ImageNet-50, ImageNet-100, ImageNet-1000, etc."""
+    binaries: bool = False
 
 
 # Possible optimizer configurations.
@@ -69,4 +70,4 @@ def train(
 
 
 if __name__ == "__main__":
-    tyro.cli(train, config=(tyro.conf.ConsolidateSubcommandArgs,))
+    tyro.cli(train, config=(tyro.conf.CascadeSubcommandArgs,))
