@@ -2,7 +2,7 @@
 #
 # Passing a Union type directly to tyro.cli() doesn't type-check correctly in
 # mypy. This will be fixed by `typing.TypeForm`: https://peps.python.org/pep-0747/
-"""Generic Subcommands
+"""Generic subcommands
 
 Just like standard classes, generic classes within unions can be selected
 between using subcommands.
