@@ -1,13 +1,15 @@
 from typing import TYPE_CHECKING
 
-__version__ = "0.9.35"
+__version__ = "1.0.0"
 
 
 from . import conf as conf
 from . import constructors as constructors
 from . import extras as extras
 from ._cli import cli as cli
+from ._settings import _experimental_options as _experimental_options
 from ._singleton import MISSING as MISSING
+from ._singleton import MISSING_NONPROP as MISSING_NONPROP
 
 # Deprecated interface.
 if not TYPE_CHECKING:
