@@ -22,6 +22,7 @@ See :doc:`/examples/basics` for examples of using configuration markers.
 from ._confstruct import arg as arg
 from ._confstruct import subcommand as subcommand
 from ._markers import AvoidSubcommands as AvoidSubcommands
+from ._markers import CascadeSubcommandArgs as CascadeSubcommandArgs
 from ._markers import ConsolidateSubcommandArgs as ConsolidateSubcommandArgs
 from ._markers import DisallowNone as DisallowNone
 from ._markers import EnumChoicesFromValues as EnumChoicesFromValues
@@ -29,6 +30,7 @@ from ._markers import Fixed as Fixed
 from ._markers import FlagConversionOff as FlagConversionOff
 from ._markers import FlagCreatePairsOff as FlagCreatePairsOff
 from ._markers import HelptextFromCommentsOff as HelptextFromCommentsOff
+from ._markers import NewSubcommandForDefaults as NewSubcommandForDefaults
 from ._markers import OmitArgPrefixes as OmitArgPrefixes
 from ._markers import OmitSubcommandPrefixes as OmitSubcommandPrefixes
 from ._markers import Positional as Positional
@@ -37,5 +39,8 @@ from ._markers import Suppress as Suppress
 from ._markers import SuppressFixed as SuppressFixed
 from ._markers import UseAppendAction as UseAppendAction
 from ._markers import UseCounterAction as UseCounterAction
+from ._markers import (
+    UsePythonSyntaxForLiteralCollections as UsePythonSyntaxForLiteralCollections,
+)
 from ._markers import configure as configure
 from ._mutex_group import create_mutex_group as create_mutex_group
