@@ -9,16 +9,18 @@ from typing import Any, Callable
 import pytest
 
 import tyro
-import tyro._arguments
+import tyro._fmtlib
 import tyro._strings
+from tyro._singleton import MISSING_NONPROP
 
 
 def get_helptext_with_checks(
     f: Callable[..., Any],
     args: list[str] = ["--help"],
     use_underscores: bool = False,
-    default: Any = None,
+    default: Any = MISSING_NONPROP,
     config: tuple[Any, ...] = (),
+    compact_help: bool = False,
 ) -> str:
     """Get the helptext for a given tyro with input, while running various
     checks along the way."""
@@ -32,32 +34,14 @@ def get_helptext_with_checks(
             default=default,
             console_outputs=False,
             config=config,
+            compact_help=compact_help,
         )
-    assert target.getvalue() == ""
+    assert target.getvalue() == "", target.getvalue()
 
     # Check tyro.extras.get_parser().
     parser = tyro.extras.get_parser(f, use_underscores=use_underscores)
     assert isinstance(parser, argparse.ArgumentParser)
 
-    # Returned parser should have formatting information stripped. External tools rarely
-    # support ANSI sequences.
-    #
-    # Note: we should check this, but the formatting information is already
-    # stripped if you run pytest without -s. We also have theming, borders,
-    # etc, that this logic doesn't currently consider.
-    #
-    # unformatted_helptext = parser.format_help()
-    # assert (
-    #     tyro._strings.strip_ansi_sequences(unformatted_helptext) == unformatted_helptext
-    # ), (
-    #     tyro._strings.strip_ansi_sequences(unformatted_helptext)
-    #     + "\n|\n"
-    #     + unformatted_helptext
-    # )
-    unformatted_usage = parser.format_usage()
-    # assert tyro._strings.strip_ansi_sequences(unformatted_usage) == unformatted_usage
-    del unformatted_usage
-
     # Basic checks for completion scripts.
     with pytest.raises(SystemExit):
         tyro.cli(
@@ -78,7 +62,8 @@ def get_helptext_with_checks(
                     # `--tyro-print-completion` is deprecated! We should use `--tyro-write-completion` instead.
                     tyro.cli(f, default=default, args=[command, shell], config=config)
             output = target.getvalue()
-            assert "shtab" in output
+            # Check that completion was generated (either by shtab or tyro).
+            assert "shtab" in output or "tyro" in output
 
     # Test with underscores
     for shell in ["bash", "zsh"]:
@@ -92,7 +77,8 @@ def get_helptext_with_checks(
                 config=config,
             )
         output = target.getvalue()
-        assert "shtab" in output
+        # Check that completion was generated (either by shtab or tyro).
+        assert "shtab" in output or "tyro" in output
 
     # Get the actual helptext.
     target = io.StringIO()
@@ -103,28 +89,13 @@ def get_helptext_with_checks(
             use_underscores=use_underscores,
             default=default,
             config=config,
+            compact_help=compact_help,
         )
+    helptext = tyro._strings.strip_ansi_sequences(target.getvalue())
 
-    # Check helptext with vs without formatting. This can help catch text wrapping bugs
-    # caused by ANSI sequences.
-    target2 = io.StringIO()
-    with pytest.raises(SystemExit), contextlib.redirect_stdout(target2):
-        tyro._arguments.USE_RICH = False
-        tyro.cli(
-            f,
-            default=default,
-            args=args,
-            use_underscores=use_underscores,
-            config=config,
-        )
-        tyro._arguments.USE_RICH = True
+    # Assert that internal implementation details are not exposed in helptext.
+    assert "__tyro-dummy-inner__" not in helptext, (
+        "Internal implementation detail '__tyro-dummy-inner__' should not appear in helptext"
+    )
 
-    if target2.getvalue() != tyro._strings.strip_ansi_sequences(target.getvalue()):
-        raise AssertionError(
-            "Potential wrapping bug! These two strings should match:\n"
-            + target2.getvalue()
-            + "\n\n"
-            + tyro._strings.strip_ansi_sequences(target.getvalue())
-        )
-
-    return target2.getvalue()
+    return helptext
