@@ -36,7 +36,7 @@ def as_callable(union_type: Any) -> Callable[..., Any]:
     return cast(Callable[..., Any], union_type)
 
 
-def test_basic_msgspec_subcommands():
+def test_basic_msgspec_subcommands() -> None:
     """Test basic msgspec subcommands with Union types."""
 
     class Checkout(msgspec.Struct):
@@ -83,7 +83,7 @@ def test_basic_msgspec_subcommands():
     assert "Commit changes" in helptext
 
 
-def test_msgspec_subcommands_with_same_type_different_generics():
+def test_msgspec_subcommands_with_same_type_different_generics() -> None:
     """Test msgspec subcommands with the same type but different generic parameters."""
 
     class Process(msgspec.Struct, Generic[T]):
@@ -107,7 +107,7 @@ def test_msgspec_subcommands_with_same_type_different_generics():
     assert result_str.data == "hello"
 
 
-def test_msgspec_mixed_dataclass_and_msgspec_subcommands():
+def test_msgspec_mixed_dataclass_and_msgspec_subcommands() -> None:
     """Test mixing dataclass and msgspec types in subcommands."""
 
     @dataclasses.dataclass
@@ -151,7 +151,7 @@ def test_msgspec_mixed_dataclass_and_msgspec_subcommands():
     assert "A command implemented as a msgspec struct" in helptext
 
 
-def test_msgspec_subcommands_with_enums():
+def test_msgspec_subcommands_with_enums() -> None:
     """Test msgspec subcommands with enum types."""
 
     class LogLevel(enum.Enum):
