@@ -75,7 +75,7 @@ def test_attrs_helptext() -> None:
         with contextlib.redirect_stdout(f):
             tyro.cli(Helptext, args=["--help"])
     helptext = f.getvalue()
-    assert tyro._strings.strip_ansi_sequences(cast(str, Helptext.__doc__)) in helptext
+    assert cast(str, Helptext.__doc__) in helptext
 
     assert "Documentation 1" in helptext
     assert "Documentation 2" in helptext
@@ -100,8 +100,7 @@ def test_attrs_next_gen_and_factory() -> None:
         with contextlib.redirect_stdout(f):
             tyro.cli(Helptext, args=["--help"])
     helptext = f.getvalue()
-    assert tyro._strings.strip_ansi_sequences(cast(str, Helptext.__doc__)) in helptext
-
+    assert cast(str, Helptext.__doc__) in helptext
     assert "Documentation 1" in helptext
     assert "Documentation 2" in helptext
     assert "Documentation 3" in helptext
@@ -155,23 +154,53 @@ def test_attrs_inheritance_with_same_typevar() -> None:
 
 def test_diamond_inheritance() -> None:
     @define(frozen=True)
-    class A:
+    class DiamondA:
         field: int | str = 5
 
     @define(frozen=True)
-    class B(A):
+    class DiamondB(DiamondA):
         pass
 
     @define(frozen=True)
-    class C(A):
+    class DiamondC(DiamondA):
         field: int = 10
 
     @define(frozen=True)
-    class D(B, C):
+    class DiamondD(DiamondB, DiamondC):
         pass
 
     # C should come earlier int the MRO than A.
-    helptext = get_helptext_with_checks(D)
-    assert "5" not in helptext
+    helptext = get_helptext_with_checks(DiamondD)
+    assert ": 5" not in helptext
     assert "10" in helptext
     assert "INT|STR" not in helptext
+
+
+def test_attrs_init_false_with_default_instance() -> None:
+    """Test that init=False fields preserve values from default instance (issue #390)."""
+
+    @attr.define
+    class AttrsConfig:
+        in_channel: int = attr.field(init=False, default=0)
+        out_channel: int = 3
+
+    # Create a default instance and set the init=False field.
+    default = AttrsConfig()
+    default.in_channel = 10
+
+    # The init=False field should preserve its value from the default instance.
+    config = tyro.cli(AttrsConfig, default=default, args=["--out-channel", "5"])
+    assert config.in_channel == 10
+    assert config.out_channel == 5
+
+    # Test with no args (should use default values).
+    config2 = tyro.cli(AttrsConfig, default=default, args=[])
+    assert config2.in_channel == 10
+    assert config2.out_channel == 3
+
+    # Test with falsy value (0) to ensure we don't skip None check incorrectly.
+    default_falsy = AttrsConfig()
+    default_falsy.in_channel = 0
+    config3 = tyro.cli(AttrsConfig, default=default_falsy, args=["--out-channel", "7"])
+    assert config3.in_channel == 0
+    assert config3.out_channel == 7
